function [im_n] = add_corr(im,sigma,type,bandwidth)
%=============================================================================
%add_corr Adds correlated noise to an image
%
%this function adds correlated noise to an image by adding filtered white
%noise with standard deviation sigma
%
%   bandwidth:  bandwidth parameter for the noise correlation (0-1)
%   type:       type of correlated noise
%               valid choices are 1-8
%   sigma:      white noise standard deviation
%   im:         input image
%   im_n:   	output image
%
% Author: Jan Aelterman, TELIN/UGent, Sept. 2008.
%==========================================================================
if nargin<4
    bandwidth=0.75;
end;
if nargin<3
    type=8;
end;

im_n=zeros(size(im));
if size(im,3)==3
    for i=1:3
        im_n(:,:,i)=add_corr_single(im(:,:,i),sigma,type,bandwidth);
    end;
else
    im_n=add_corr_single(im,sigma,type,bandwidth);
end;
end

function [im_n] = add_corr_single(im,sigma,type,bandwidth)
sx=size(im,2);
sy=size(im,1);
halfy=floor(bandwidth*sy);
halfx=floor(bandwidth*sx);
halfy=halfy-mod(halfy,2)-mod(sy+1,2);
halfx=halfx-mod(halfx,2)-mod(sx+1,2);
noise=sigma*randn(size(im));
noise_spectrum=fftshift(fft2(noise));
switch lower(type)
    case 1
        corr_noise_spectrum=noise_spectrum;
        corr_noise_spectrum(:,[1:ceil(halfx/2),end-floor(halfx/2)+1:end])=0;
        noise_spectrum=noise_spectrum-corr_noise_spectrum;
    case 2
        noise_spectrum(:,[1:ceil(halfx/2),end-floor(halfx/2)+1:end])=0;
    case 3
        corr_noise_spectrum=noise_spectrum;
        corr_noise_spectrum([1:ceil(halfy/2),end-floor(halfy/2)+1:end],[1:ceil(halfx/2),end-floor(halfx/2)+1:end])=0;
        noise_spectrum=noise_spectrum-corr_noise_spectrum;
    case 4
        noise_spectrum([1:ceil(halfy/2),end-floor(halfy/2)+1:end],[1:ceil(halfx/2),end-floor(halfx/2)+1:end])=0;
    case 5
        corr_noise_spectrum=noise_spectrum;
        if ceil(halfy/2)~=floor(halfy/2)
            corr_noise_spectrum(1,:)=0;
        end;
        if ceil(halfx/2)~=floor(halfx/2)
            corr_noise_spectrum(:,1)=0;
        end;
        corr_noise_spectrum([1:ceil(halfy/2)],[1:ceil(halfx/2)])=0;
        corr_noise_spectrum(end-floor(halfy/2)+1:end,end-floor(halfx/2)+1:end)=0;
        noise_spectrum=noise_spectrum-corr_noise_spectrum;
    case 6
        if ceil(halfy/2)~=floor(halfy/2)
            noise_spectrum(1,:)=0;
        end;
        if ceil(halfx/2)~=floor(halfx/2)
            noise_spectrum(:,1)=0;
        end;
        noise_spectrum([1:ceil(halfy/2)],[1:ceil(halfx/2)])=0;
        noise_spectrum(end-floor(halfy/2)+1:end,end-floor(halfx/2)+1:end)=0;
    case 7
        noise_spectrum(:,[1:ceil(halfx/2),end-floor(halfx/2)+1:end])=0;
        noise_spectrum([1:ceil(halfy/2),end-floor(halfy/2)+1:end],:)=0;
    case 8

    otherwise
        disp('invalid choice');
        im_n=0;
        return;
end
im_n=double(im)+ifft2(ifftshift(noise_spectrum));
end